#!/usr/bin/env python
#
# Weather Board MQTT publisher
# tim@rowledge.org
# hacked out of SwitchDoc Weather Board support code, a paho/mqtt example and pure desperation
#
#

# imports

import sys
import time
from datetime import datetime
import random 
import paho.mqtt.publish as publish
import config
import subprocess
import RPi.GPIO as GPIO
import smbus

sys.path.append('/home/pi/SDL_Pi_WeatherBoard-master/RTC_SDL_DS3231')
sys.path.append('/home/pi/SDL_Pi_WeatherBoard-master/Adafruit_Python_BMP')
sys.path.append('/home/pi/SDL_Pi_WeatherBoard-master/Adafruit_Python_GPIO')
sys.path.append('/home/pi/SDL_Pi_WeatherBoard-master/SDL_Pi_WeatherRack')


import SDL_DS3231
import Adafruit_BMP.BMP280 as BMP280
import SDL_Pi_WeatherRack as SDL_Pi_WeatherRack



################
# Device Present State Variables
###############


config.DS3231_Present = False
config.BMP280_Present = False

config.AM2315_Present = False
config.ADS1015_Present = False
config.ADS1115_Present = False



def returnStatusLine(device, state):

        returnString = device
        if (state == True):
                returnString = returnString + ": Present" 
        else:
                returnString = returnString + ": Not Present"
        return returnString



###############   

#WeatherRack Weather Sensors
#
# GPIO Numbering Mode GPIO.BCM
#

anemometerPin = 26
rainPin = 21

# constants

SDL_MODE_INTERNAL_AD = 0
SDL_MODE_I2C_ADS1015 = 1    # internally, the library checks for ADS1115 or ADS1015 if found

#sample mode means return immediately.  THe wind speed is averaged at sampleTime or when you ask, whichever is longer
SDL_MODE_SAMPLE = 0
#Delay mode means to wait for sampleTime and the average after that time.
SDL_MODE_DELAY = 1

ReadingInterval = 10 # ie 6 second for testing, set to 10 or so for production

weatherStation = SDL_Pi_WeatherRack.SDL_Pi_WeatherRack(anemometerPin, rainPin, 0,0, SDL_MODE_I2C_ADS1015)

weatherStation.setWindMode(SDL_MODE_SAMPLE, 5.0)
#weatherStation.setWindMode(SDL_MODE_DELAY, 5.0)

################


# DS3231/AT24C32 Setup

starttime = datetime.utcnow()
dayNumber = datetime.day #so we can work out when the day has changed

ds3231 = SDL_DS3231.SDL_DS3231(1, 0x68)


try:

        #comment out the next line after the clock has been initialized
        #ds3231.write_now()
        print "DS3231=\t\t%s" % ds3231.read_datetime()
        config.DS3231_Present = True
        for x in range(0,4):
                value = random.randint(0,255)
                ds3231.write_AT24C32_byte(x, value)

except IOError as e:
        #    print "I/O error({0}): {1}".format(e.errno, e.strerror)
        config.DS3231_Present = False
        # do the AT24C32 eeprom

        
################

# BMP280 Setup

try:
        bmp280 = BMP280.BMP280()
        config.BMP280_Present = True

except IOError as e:

        #    print "I/O error({0}): {1}".format(e.errno, e.strerror)
        config.BMP280_Present = False

################



# Detect AM2315 
try:
        from tentacle_pi.AM2315 import AM2315
        try:
                am2315 = AM2315(0x5c,"/dev/i2c-1")
                temperature, humidity, crc_check = am2315.sense()
                config.AM2315_Present = True
                if (crc_check == -1):
                        config.AM2315_Present = False
        except:
                config.AM2315_Present = False
except:
        config.AM2315_Present = False
        #print "------> See Readme to install tentacle_pi"

###########


# Main Loop - sleeps 10 seconds
# Tests all I2C and WeatherRack devices on Weather Board 


# Main Program
# initialisation
totalRain = 0
MQTTBroker = "Amtiskaw"

print "Program Started at:"+ time.strftime("%Y-%m-%d %H:%M:%S")
publish.single("weatherstation/status/start", time.strftime("%Y-%m-%d %H:%M:%S"), hostname= MQTTBroker)
print returnStatusLine("DS3231",config.DS3231_Present)
publish.single("weatherstation/status/sensor", returnStatusLine("DS3231",config.DS3231_Present), hostname= MQTTBroker)
print returnStatusLine("BMP280",config.BMP280_Present)
publish.single("weatherstation/status/sensor", returnStatusLine("BMP280",config.BMP280_Present), hostname= MQTTBroker)
print returnStatusLine("AM2315",config.AM2315_Present)
publish.single("weatherstation/status/sensor", returnStatusLine("AM2315",config.AM2315_Present), hostname= MQTTBroker)
print returnStatusLine("ADS1015",config.ADS1015_Present)
publish.single("weatherstation/status/sensor", returnStatusLine("ADS1015",config.ADS1015_Present), hostname= MQTTBroker)
print returnStatusLine("ADS1115",config.ADS1115_Present)
publish.single("weatherstation/status/sensor", returnStatusLine("ADS1115",config.ADS1115_Present), hostname= MQTTBroker)


while True:
        if (datetime.today().day != dayNumber):
                totalRain = 0
                dayNumber = datetime.today().day

        try:
                if (config.DS3231_Present == True):
                        print "Raspberry Pi time =\t" + time.strftime("%Y-%m-%d %H:%M:%S")
                        print "DS3231 time =\t\t%s" % ds3231.read_datetime()
                        publish.single("weatherstation/RTCtime", ds3231.read_str(), hostname= MQTTBroker)

                        print "DS3231 Temperature= \t%0.2f C" % ds3231.getTemp()
                        publish.single("weatherstation/RTCtemp", ds3231.getTemp(), hostname= MQTTBroker)

                if (config.AM2315_Present):
                        temperature, humidity, crc_check = am2315.sense()
                        print "AM2315 temperature: %0.1f" % temperature
                        publish.single("weatherstation/outdoortemp", "%0.1f" % temperature, hostname= MQTTBroker)
                        print "AM2315 humidity: %0.1f" % humidity
                        publish.single("weatherstation/outdoorhumidity", "%0.1f" % humidity, hostname= MQTTBroker)
                        print "AM2315 crc: %s" % crc_check
                        publish.single("weatherstation/outdoorCRC", crc_check, hostname= MQTTBroker)

                currentWindSpeed = weatherStation.current_wind_speed()
                currentWindGust = weatherStation.get_wind_gust()
                totalRain = totalRain + weatherStation.get_current_rain_total()
                print("Rain Total=\t%0.2f mm")%(totalRain)
                publish.single("weatherstation/raintotal", "%0.2f" % totalRain, hostname= MQTTBroker)
                print("Wind Speed=\t%0.2f Km/Hr")%(currentWindSpeed)
                publish.single("weatherstation/windspeed", "%0.2f" % currentWindSpeed, hostname= MQTTBroker)
                print("MPH wind_gust=\t%0.2f Km/Hr")%(currentWindGust)
                publish.single("weatherstation/windgust", "%0.2f" % currentWindGust, hostname= MQTTBroker)
                if (config.ADS1015_Present or config.ADS1115_Present):  
                        print "Wind Direction=\t\t\t %0.2f Degrees" % weatherStation.current_wind_direction()
                        publish.single("weatherstation/winddirection", "%0.2f" % weatherStation.current_wind_direction(), hostname= MQTTBroker)
                        print "Wind Direction Voltage=\t\t %0.3f V" % weatherStation.current_wind_direction_voltage()
                                
         
                if (config.BMP280_Present):
                        print 'Temperature = \t{0:0.2f} C'.format(bmp280.read_temperature())
                        publish.single("weatherstation/boardtemp", "%0.1f" % bmp280.read_temperature(), hostname= MQTTBroker)
                        print 'Pressure = \t{0:0.2f} KPa'.format(bmp280.read_pressure()/1000)
                        publish.single("weatherstation/pressure", "%0.2f" % (bmp280.read_pressure()/1000), hostname= MQTTBroker)
                        #print 'Altitude = \t{0:0.2f} m'.format(bmp280.read_altitude())
                        #print 'Sealevel Pressure = \t{0:0.2f} KPa'.format(bmp280.read_sealevel_pressure()/1000)
        except IOError as e:
                print "some kind of error({0}): {1}".format(e.errno, e.strerror)
        print "Sleeping %s seconds" % (60.0 * ReadingInterval )
        time.sleep(60.0 * ReadingInterval )


